from viggocore.common import exception
from decimal import Decimal


# verifica se um número é par
def se_e_par(n):
    n = int(n)
    if (n % 2) == 0:
        return True
    else:
        return False

# função criada para arredondar um valor usando a norma ABNT 5891/77
# onde valor=o valor a ser arredondado
# n=limitador da quantidade de casas
def round_abnt(valor, n: int):
    try:
        vl_str = str(valor)
        if '.' in vl_str:
            inteira, decimal = str(valor).split('.')
        else:
            inteira = str(valor)
            decimal = '00'
    except Exception:
        print('Erro na conversão do valor!')

    decimal = (decimal + ('0' * n))
    if len(decimal) > (n+2):
        prox = int(decimal[n])
        pos_prox = int(decimal[n+1])
        if ((prox >= 5 and pos_prox != 0) or
           (prox == 5 and pos_prox == 0 and se_e_par(decimal[n]) is False)):
            aux = int(decimal[n-1]) + 1
            decimal = decimal[:n-1] + str(aux) + decimal[n:]
    decimal = decimal[:n]
    return Decimal(f'{inteira}.{decimal}')


from decimal import Decimal
from viggofiscal.subsystem.calculo_fiscal.utils import round_abnt


class BaseReduzidaIcmsProprio():

    def __init__(self, valor_produto: Decimal, valor_frete: Decimal,
                 valor_seguro: Decimal, despesas_acessorias: Decimal,
                 valor_desconto: Decimal, percentual_reducao: Decimal,
                 valor_ipi: Decimal=Decimal('0.0')):
        self.valor_produto = valor_produto 
        self.valor_frete = valor_frete
        self.valor_seguro = valor_seguro
        self.despesas_acessorias = despesas_acessorias
        self.valor_desconto = valor_desconto
        self.percentual_reducao = percentual_reducao
        self.valor_ipi = valor_ipi
    
    def calcular_base_reduzida_icms_proprio(self) -> Decimal:
        base_icms = (
            self.valor_produto + self.valor_frete + self.valor_seguro +
            self.despesas_acessorias - self.valor_desconto)
        base_reduzida_icms_proprio = (
            base_icms - (base_icms * (self.percentual_reducao / 100)) +
            self.valor_ipi)
        return round_abnt(base_reduzida_icms_proprio, 2)



from decimal import Decimal
from viggofiscal.subsystem.calculo_fiscal.icms.base_icms_proprio \
    import BaseIcmsProprio
from viggofiscal.subsystem.calculo_fiscal.icms.valor_icms_proprio \
    import ValorIcmsProprio

class Icms00():

    def __init__(self, valor_produto: Decimal, valor_frete: Decimal,
                 valor_seguro: Decimal, despesas_acessorias: Decimal,
                 valor_ipi: Decimal, valor_desconto: Decimal,
                 aliquota_icms_proprio: Decimal):
        self.valor_produto = valor_produto
        self.valor_frete = valor_frete
        self.valor_seguro = valor_seguro
        self.despesas_acessorias = despesas_acessorias
        self.valor_ipi = valor_ipi
        self.valor_desconto = valor_desconto
        self.aliquota_icms_proprio = aliquota_icms_proprio
        self.base_icms = BaseIcmsProprio(
            valor_produto, valor_frete, valor_seguro, despesas_acessorias,
            valor_desconto, valor_ipi)
    
    def base_icms_proprio(self):
        return self.base_icms.calcular_base_icms_proprio()
    
    def valor_icms_proprio(self) -> Decimal:
        return ValorIcmsProprio(
            self.base_icms_proprio(), self.aliquota_icms_proprio).\
            calcular_valor_icms_proprio()



from decimal import Decimal
from viggofiscal.subsystem.calculo_fiscal.icms.base_reduzida_icms_proprio \
    import BaseReduzidaIcmsProprio
from viggofiscal.subsystem.calculo_fiscal.icms.icms00 \
    import Icms00
from viggofiscal.subsystem.calculo_fiscal.utils import round_abnt

class Icms20():

    def __init__(self, valor_produto: Decimal, valor_frete: Decimal,
                 valor_seguro: Decimal, despesas_acessorias: Decimal,
                 valor_ipi: Decimal, valor_desconto: Decimal,
                 aliquota_icms_proprio: Decimal, percentual_reducao: Decimal):
        self.valor_produto = valor_produto
        self.valor_frete = valor_frete
        self.valor_seguro = valor_seguro
        self.despesas_acessorias = despesas_acessorias
        self.valor_ipi = valor_ipi
        self.valor_desconto = valor_desconto
        self.aliquota_icms_proprio = aliquota_icms_proprio
        self.percentual_reducao = percentual_reducao
        self.base_reduzida_icms = BaseReduzidaIcmsProprio(
            valor_produto, valor_frete, valor_seguro, despesas_acessorias,
            valor_desconto, percentual_reducao, valor_ipi)
    
    def base_reduzida_icms_proprio(self):
        return self.base_reduzida_icms.calcular_base_reduzida_icms_proprio()
    
    def valor_icms_proprio(self) -> Decimal:
        base_reduzida_icms = self.base_reduzida_icms_proprio()
        valor_icms = base_reduzida_icms * (self.aliquota_icms_proprio / 100)
        return round_abnt(valor_icms, 2)

    def valor_icms_desonerado(self) -> Decimal:
        icms00 = Icms00(
            self.valor_frete, self.valor_seguro, self.despesas_acessorias,
            self.valor_ipi, self.valor_desconto, self.aliquota_icms_proprio)
        valor_icms_normal = icms00.valor_icms_proprio()
        valor_icms_desonerado = valor_icms_normal - self.valor_icms_proprio()
        return round_abnt(valor_icms_desonerado, 2)